module cosmonet
use cmbtypes
  use CAMB, only : CAMB_GetResults, CAMB_GetAge, CAMBParams, CAMB_SetDefParams,Transfer_GetMatterPower, &
          AccuracyBoost,  Cl_scalar, Cl_tensor, Cl_lensed, outNone, w_lam, &
          CAMBParams_Set,Re_OpticalDepthAtZ, MT, CAMBdata, NonLinear_Pk,&
          CAMB_GetTransfers,CAMB_FreeCAMBdata,CAMB_InitCAMBdata, CAMB_TransfersToPowers, &
          initial_adiabatic,initial_vector,initial_iso_baryon,initial_iso_neutrino, initial_iso_neutrino_vel
  implicit none

	


#define NLAYERS 3

real, allocatable :: weightsTT(:), weightsTE(:), weightsEE(:), weightsMPK(:)
integer, allocatable :: labelsTT(:), labelsTE(:), labelsEE(:), labelsMPK(:)

!!!!!!!!!!!!!!!!!!!!!SPECTRA WEIGHTS FILES!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

character(LEN=80) :: WeightFILETT = "tt.prm"
character(LEN=80) :: WeightFILETE = "te.prm"
character(LEN=80) :: WeightFILEEE = "ee.prm"
character(LEN=80) :: WeightFILEMPK = "mpt.prm"
character(LEN=80) :: NETSTR_FILETT = "tt_netstr.prm"
character(LEN=80) :: NETSTR_FILETE = "te_netstr.prm"
character(LEN=80) :: NETSTR_FILEEE = "ee_netstr.prm"
character(LEN=80) :: NETSTR_FILEMPK = "mpt_netstr.prm"
character(LEN=80) :: WeightFILECLASS = "classification.prm"
character(LEN=80) :: NETSTR_FILECLASS = "classification_netstr.prm"


!!!!!!!!!!!!!!!!!!!!!LIKELIHOOD WEIGHTS FILES!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

character(LEN=80) :: NETSTR_LIKE_WMAP_FILE = "lhood1_netstr.prm"
character(LEN=80) :: PARAM_LIKE_WMAP_FILE = "lhood1.prm"
character(LEN=80) :: NETSTR_LIKE_MPK1_FILE = "lhood2_netstr.prm"
character(LEN=80) :: PARAM_LIKE_MPK1_FILE = "lhood2.prm"
character(LEN=80) :: NETSTR_LIKE_MPK2_FILE = "lhood3_netstr.prm"
character(LEN=80) :: PARAM_LIKE_MPK2_FILE = "lhood3.prm"


!!!!!!!!!!!!!!!!!!!!!INTERPOLATION LIMITS FOR EACH PARAMETER!!!!!!!!!!!!!!!!

character(LEN=80) :: NETPARAMS = "net_params.dat"

!!!IF YOU WANT ONLY COSMONET SPECTRA SET user_cosmonet = .true. AND user_cosmonet_like = .false. !!!!
!!!IF YOU WANT COSMONET LIKELIHOOD SET BOTH TO .true. !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

logical :: user_cosmonet = .true.
logical :: user_cosmonet_like = .true.

contains




subroutine init_cambnetWMAPLike(time)
!use global
use cmbtypes
integer l(1:60)
integer paramlike(1:8)                                                 
integer n
logical time

time = .false.            
                                                                                                                                                
open(43, FILE = NETSTR_LIKE_WMAP_FILE)
read(43, *) paramlike
                                                                                                                                    
WMAPnetlike%nlayers = paramlike(1)
WMAPnetlike%nin = paramlike(2)
WMAPnetlike%nhid = paramlike(3)
WMAPnetlike%nhid2 = paramlike(4)
WMAPnetlike%nout = paramlike(5)
WMAPnetlike%nweights = paramlike(6)
WMAPnetlike%nparams = paramlike(7)
WMAPnetlike%nnodes = paramlike(8)


open(444, FILE = PARAM_LIKE_WMAP_FILE)
read(444,*) WMAPweightslike

write(*,*) "done with initWMAPlike"
                                                                                                                                                            
end  subroutine init_cambnetWMAPLike






subroutine init_cambnetMPK1Like(time)
!use global
use cmbtypes
integer l(1:60)
integer paramlike(1:8)                                                 
integer n
logical time

time = .false.            
                                                                                                                                                
open(434, FILE = NETSTR_LIKE_MPK1_FILE)
read(434, *) paramlike
                                                                                                                                    
MPK1netlike%nlayers = paramlike(1)
MPK1netlike%nin = paramlike(2)
MPK1netlike%nhid = paramlike(3)
MPK1netlike%nhid2 = paramlike(4)
MPK1netlike%nout = paramlike(5)
MPK1netlike%nweights = paramlike(6)
MPK1netlike%nparams = paramlike(7)
MPK1netlike%nnodes = paramlike(8)

open(445, FILE = PARAM_LIKE_MPK1_FILE)
read(445,*) MPK1weightslike

write(*,*) "done with initMPK1like"
                                                                                                                                                            
end  subroutine init_cambnetMPK1Like

subroutine init_cambnetMPK2Like(time)
!use global
use cmbtypes
integer l(1:60)
integer paramlike(1:8)                                                 
integer n
logical time

time = .false.            
                                                                                                                                                
open(435, FILE = NETSTR_LIKE_MPK2_FILE)
read(435, *) paramlike
                                                                                                                                    
MPK2netlike%nlayers = paramlike(1)
MPK2netlike%nin = paramlike(2)
MPK2netlike%nhid = paramlike(3)
MPK2netlike%nhid2 = paramlike(4)
MPK2netlike%nout = paramlike(5)
MPK2netlike%nweights = paramlike(6)
MPK2netlike%nparams = paramlike(7)
MPK2netlike%nnodes = paramlike(8)

open(4445, FILE = PARAM_LIKE_MPK2_FILE)
read(4445,*) MPK2weightslike

write(*,*) "done with initMPK2like"
                                                                                                                                                            
end  subroutine init_cambnetMPK2Like


 subroutine init_cambnet(netTT, netTE, netEE, netMPK, weightsTT, weightsTE,weightsEE, weightsMPK, labelsTT, labelsTE, labelsEE, labelsMPK, time)
use cmbtypes
Type(NetStr)  netTT, netTE, netEE, netMPK
real, allocatable :: weightsTT(:)
real, allocatable :: weightsTE(:)
real, allocatable :: weightsEE(:)
real, allocatable :: weightsMPK(:)
integer l(1:60)
integer, allocatable :: labelsTT(:)
integer, allocatable :: labelsTE(:)
integer, allocatable :: labelsEE(:)
integer, allocatable :: labelsMPK(:)
integer paramTT(1:8)
integer paramTE(1:8)
integer paramEE(1:8)
integer paramMPK(1:8)
logical time
integer n
time = .false.
!!!TT
open(13, FILE = NETSTR_FILETT)
read(13, *) paramTT

open(23, FILE = NETSTR_FILETE)
read(23, *) paramTE

open(33, FILE = NETSTR_FILEEE)
read(33, *) paramEE

open(33, FILE = NETSTR_FILEMPK)
read(33, *) paramMPK

netTT%nlayers = paramTT(1)
netTT%nin = paramTT(2)
netTT%nhid = paramTT(3)
netTT%nhid2 = paramTT(4)
netTT%nout = paramTT(5)
netTT%nweights = paramTT(6)
netTT%nparams = paramTT(7)
netTT%nnodes = paramTT(8)

netTE%nlayers = paramTE(1)
netTE%nin = paramTE(2)
netTE%nhid = paramTE(3)
netTE%nhid2 = paramTE(4)
netTE%nout = paramTE(5)
netTE%nweights = paramTE(6)
netTE%nparams = paramTE(7)
netTE%nnodes = paramTE(8)

netEE%nlayers = paramEE(1)
netEE%nin = paramEE(2)
netEE%nhid = paramEE(3)
netEE%nhid2 = paramEE(4)
netEE%nout = paramEE(5)
netEE%nweights = paramEE(6)
netEE%nparams = paramEE(7)
netEE%nnodes = paramEE(8)

netMPK%nlayers = paramMPK(1)
netMPK%nin = paramMPK(2)
netMPK%nhid = paramMPK(3)
netMPK%nhid2 = paramMPK(4)
netMPK%nout = paramMPK(5)
netMPK%nweights = paramMPK(6)
netMPK%nparams = paramMPK(7)
netMPK%nnodes = paramMPK(8)

open(14, FILE = WeightFILETT)
allocate(weightsTT(1:netTT%nparams))
read(14,*) weightsTT

open(24, FILE = WeightFILETE)
allocate(weightsTE(1:netTE%nparams))
read(24,*) weightsTE

open(34, FILE = WeightFILEEE)
allocate(weightsEE(1:netEE%nparams))
read(34,*) weightsEE

open(34, FILE = WeightFILEMPK)
allocate(weightsMPK(1:netMPK%nparams))
read(34,*) weightsMPK

write(*,*) "INIT FINISHED"

end  subroutine init_cambnet


subroutine init_classnet(time)
!use global
use cmbtypes
integer l(1:60)
integer paramlike(1:8)                                                 
integer n
logical time
Type(NetStr) netclass
time = .false.            
                                                                                                                                                
open(430, FILE = NETSTR_FILECLASS)
read(430, *) paramlike
                                                                                                                                    
netclass%nlayers = paramlike(1)
netclass%nin = paramlike(2)
netclass%nhid = paramlike(3)
netclass%nhid2 = paramlike(4)
netclass%nout = paramlike(5)
netclass%nweights = paramlike(6)
netclass%nparams = paramlike(7)
netclass%nnodes = paramlike(8)


open(1, FILE = WeightFILECLASS)
read(1,*) classweights

write(*,*) "done with initclassnet"
                                                                                                                                                            
end  subroutine init_classnet

subroutine regress_classnet(P, classweights, net, class)
use cmbtypes
integer i,j,  num_Cls, lmax
Type(NetStr) net
type(CAMBParams)  P
real class(1:2)







real parameters(1:7)
!real weights(1:net%nweights)
!real scalein(1:net%nin)
!real offin(1:net%nin)
!real scaleout(1:net%nout)
!real offout(1:net%nout)
!real inputs(1:net%nin)
!real hidden1(1:net%nhid)
!real outputs(1:net%nout)
!real theta1(1:net%nhid)
!real theta2(1:net%nout)
!real w1(1:net%nin*net%nhid)
!real w2(1:net%nhid*net%nout)


real classweights(1:502)
real scalein(1:7)
real offin(1:7)
real scaleout(1:2)
real offout(1:2)
real inputs(1:7)
real hidden1(1:50)
real outputs(1:2)
real theta1(1:50)
real theta2(1:2)
real w1(1:7*50)
real w2(1:50*2)


net%nweights = 502
net%nin = 7
net%nout = 2
net%nhid = 50



parameters(1) = COSMONETP(5)
parameters(2) = COSMONETP(2)
parameters(3) = COSMONETP(1)
parameters(4) = COSMONETP(3)
parameters(5) = COSMONETP(4)
parameters(6) = COSMONETP(6)
parameters(7) = COSMONETP(7)

write(*,*) "parameters", parameters
!parameters(1) = -0.04
!parameters(2) = -0.75
!parameters(3) = 0.111
!write(*,*) "class parameters", parameters
      j=1
        do i=1, net%nhid
                theta1(i)=classweights(j);
                j=j+1
	end do
!write(*,*) "tt_theta1", theta1

        do i=1, net%nin*net%nhid
                w1(i)=classweights(j);
                j=j+1
	end do

!write(*,*) "tt_w1", w1
	do i=1, net%nout
                theta2(i)=classweights(j);
                j=j+1
	end do
!write(*,*) "tt_theta2", theta2       
	do i=1, net%nhid*net%nout
                w2(i)=classweights(j);
                j=j+1
        end do

!! apply feedforward calculation
!! hidden layer:
        do i=1, net%nhid
              hidden1(i)=theta1(i)
                do j=1, net%nin
                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + (i-1)*net%nin))
		end do
                hidden1(i)=tanh(hidden1(i))
        end do
!write(*,*) "tt_hidden1", hidden1
!! output layer
	do i=1, net%nout
                outputs(i)=theta2(i)
		do j=1, net%nhid
                        outputs(i) = outputs(i) + (hidden1(j)*w2(j+(i-1)*net%nhid))
		end do
	end do
!write(*,*) "tt_outputs", outputs
!! scale the results
	do i=1, net%nout
                class(i) = outputs(i)
	end do

!write(*,*) class

end subroutine 



function regress_WMAP_like()
use cmbtypes
real regress_WMAP_like
integer lmax, num_Cls, i,j
real parameters(1:WMAPnetlike%nin)


real scalein(1:WMAPnetlike%nin)
real offin(1:WMAPnetlike%nin)
real scaleout(1:WMAPnetlike%nout)
real offout(1:WMAPnetlike%nout)
real inputs(1:WMAPnetlike%nin)
real hidden1(1:WMAPnetlike%nhid)
real outputs(1:WMAPnetlike%nout)
real theta1(1:WMAPnetlike%nhid)
real theta2(1:WMAPnetlike%nout)
real w1(1:WMAPnetlike%nin*WMAPnetlike%nhid)
real w2(1:WMAPnetlike%nhid*WMAPnetlike%nout)

!write(*,*) "WMAPnetlike%nin", WMAPnetlike%nin
do i=1, WMAPnetlike%nin
	parameters(i) = COSMONETP(i)
end do
!write(*,*) "parameters", parameters                    

     j=1
        do i=1, WMAPnetlike%nhid
                theta1(i)=WMAPweightslike(j);
                j=j+1
        end do

                                                                                 
        do i=1, WMAPnetlike%nin*WMAPnetlike%nhid
                w1(i)=WMAPweightslike(j);
                j=j+1
        end do


        do i=1, WMAPnetlike%nout
                theta2(i)=WMAPweightslike(j);
                j=j+1
        end do


        do i=1, WMAPnetlike%nhid*WMAPnetlike%nout
                w2(i)=WMAPweightslike(j);
                j=j+1
        end do


!! apply feedforward calculation
!! hidden layer:
!        do i=1, WMAPnetlike%nhid
!              hidden1(i)=theta1(i)
!                do j=1, WMAPnetlike%nin
!                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + i*WMAPnetlike%nin))
!                end do
!                hidden1(i)=tanh(hidden1(i))
!        end do
do i=1, WMAPnetlike%nhid
	hidden1(i) = theta1(i)
	do j=1, WMAPnetlike%nin
		hidden1(i) = hidden1(i) + (parameters(j)*w1(j+(i-1)*WMAPnetlike%nin))
	end do
	hidden1(i) = tanh(hidden1(i))
end do
!write(*,*) "hidden1", hidden1
outputs = 0d0
!! output layer
!        do i=1, WMAPnetlike%nout
!                outputs(i)=theta2(i)
!                do j=1, WMAPnetlike%nhid
!                        outputs(i) = outputs(i) + & 
!			 (hidden1(j)*w2( j+i*WMAPnetlike%nhid ))
!                end do
!        end do
outputs(1) = theta2(1)
	do j=1, WMAPnetlike%nhid
		outputs(1) = outputs(1) + (hidden1(j)*w2( j ))
	end do

!write(*,*) "outputs", outputs
!! scale the results
        do i=1, WMAPnetlike%nout
                regress_WMAP_like = outputs(i)
		!write(*,*) "regress_like", regress_like
	end do




end function regress_WMAP_like







function regress_MPK1_like()
use cmbtypes
real regress_MPK1_like
integer lmax, num_Cls, i,j
real parameters(1:num_params_used)


real scalein(1:MPK1netlike%nin)
real offin(1:MPK1netlike%nin)
real scaleout(1:MPK1netlike%nout)
real offout(1:MPK1netlike%nout)
real inputs(1:MPK1netlike%nin)
real hidden1(1:MPK1netlike%nhid)
real outputs(1:MPK1netlike%nout)
real theta1(1:MPK1netlike%nhid)
real theta2(1:MPK1netlike%nout)
real w1(1:MPK1netlike%nin*MPK1netlike%nhid)
real w2(1:MPK1netlike%nhid*MPK1netlike%nout)

do i=1, 10
parameters(i) = COSMONETP(i)
end do
                    

     j=1
        do i=1, MPK1netlike%nhid
                theta1(i)=MPK1weightslike(j);
                j=j+1
        end do

                                                                                 
        do i=1, MPK1netlike%nin*MPK1netlike%nhid
                w1(i)=MPK1weightslike(j);
                j=j+1
        end do


        do i=1, MPK1netlike%nout
                theta2(i)=MPK1weightslike(j);
                j=j+1
        end do


        do i=1, MPK1netlike%nhid*MPK1netlike%nout
                w2(i)=MPK1weightslike(j);
                j=j+1
        end do


!! apply feedforward calculation
!! hidden layer:
!        do i=1, MPK1netlike%nhid
!              hidden1(i)=theta1(i)
!                do j=1, MPK1netlike%nin
!                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + i*MPK1netlike%nin))
!                end do
!                hidden1(i)=tanh(hidden1(i))
!        end do
do i=1, MPK1netlike%nhid
	hidden1(i) = theta1(i)
	do j=1, MPK1netlike%nin
		hidden1(i) = hidden1(i) + (parameters(j)*w1(j+(i-1)*MPK1netlike%nin))
	end do
	hidden1(i) = tanh(hidden1(i))
end do
!write(*,*) "hidden1", hidden1
outputs = 0d0
!! output layer
!        do i=1, MPK1netlike%nout
!                outputs(i)=theta2(i)
!                do j=1, MPK1netlike%nhid
!                        outputs(i) = outputs(i) + & 
!			 (hidden1(j)*w2( j+i*MPK1netlike%nhid ))
!                end do
!        end do
outputs(1) = theta2(1)
	do j=1, MPK1netlike%nhid
		outputs(1) = outputs(1) + (hidden1(j)*w2( j ))
	end do

!write(*,*) "outputs", outputs
!! scale the results
        do i=1, MPK1netlike%nout
                regress_MPK1_like = outputs(i)
		!write(*,*) "regress_like", regress_like
	end do




end function regress_MPK1_like


function regress_MPK2_like()
use cmbtypes
real regress_MPK2_like
integer lmax, num_Cls, i,j
real parameters(1:num_params_used)


real scalein(1:MPK2netlike%nin)
real offin(1:MPK2netlike%nin)
real scaleout(1:MPK2netlike%nout)
real offout(1:MPK2netlike%nout)
real inputs(1:MPK2netlike%nin)
real hidden1(1:MPK2netlike%nhid)
real outputs(1:MPK2netlike%nout)
real theta1(1:MPK2netlike%nhid)
real theta2(1:MPK2netlike%nout)
real w1(1:MPK2netlike%nin*MPK1netlike%nhid)
real w2(1:MPK2netlike%nhid*MPK1netlike%nout)

do i=1, num_params_used
parameters(i) = COSMONETP(i)
end do
                    

     j=1
        do i=1, MPK2netlike%nhid
                theta1(i)=MPK2weightslike(j);
                j=j+1
        end do

                                                                                 
        do i=1, MPK2netlike%nin*MPK2netlike%nhid
                w1(i)=MPK2weightslike(j);
                j=j+1
        end do


        do i=1, MPK2netlike%nout
                theta2(i)=MPK2weightslike(j);
                j=j+1
        end do


        do i=1, MPK2netlike%nhid*MPK2netlike%nout
                w2(i)=MPK2weightslike(j);
                j=j+1
        end do


!! apply feedforward calculation
!! hidden layer:
!        do i=1, MPK2netlike%nhid
!              hidden1(i)=theta1(i)
!                do j=1, MPK2netlike%nin
!                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + i*MPK2netlike%nin))
!                end do
!                hidden1(i)=tanh(hidden1(i))
!        end do
do i=1, MPK2netlike%nhid
	hidden1(i) = theta1(i)
	do j=1, MPK2netlike%nin
		hidden1(i) = hidden1(i) + (parameters(j)*w1(j+(i-1)*MPK2netlike%nin))
	end do
	hidden1(i) = tanh(hidden1(i))
end do
!write(*,*) "hidden1", hidden1
outputs = 0d0
!! output layer
!        do i=1, MPK2netlike%nout
!                outputs(i)=theta2(i)
!                do j=1, MPK2netlike%nhid
!                        outputs(i) = outputs(i) + & 
!			 (hidden1(j)*w2( j+i*MPK2netlike%nhid ))
!                end do
!        end do
outputs(1) = theta2(1)
	do j=1, MPK2netlike%nhid
		outputs(1) = outputs(1) + (hidden1(j)*w2( j ))
	end do

!write(*,*) "outputs", outputs
!! scale the results
        do i=1, MPK2netlike%nout
                regress_MPK2_like = outputs(i)
		!write(*,*) "regress_like", regress_like
	end do




end function regress_MPK2_like

subroutine regress_cambnetMPK(P, weights, net, MPK)
use cmbtypes
integer i,j,  num_Cls, lmax
Type(NetStr) net
type(CAMBParams)  P
real MPK(1:153)






real parameters(1:num_params_used)
real weights(1:net%nweights)
real scalein(1:net%nin)
real offin(1:net%nin)
real scaleout(1:net%nout)
real offout(1:net%nout)
real inputs(1:net%nin)
real hidden1(1:net%nhid)
real outputs(1:net%nout)
real theta1(1:net%nhid)
real theta2(1:net%nout)
real w1(1:net%nin*net%nhid)
real w2(1:net%nhid*net%nout)



do i=1, 7
parameters(i) = COSMONETP(i)
end do


      j=1
        do i=1, net%nhid
                theta1(i)=weights(j);
                j=j+1
	end do
!write(*,*) "tt_theta1", theta1

        do i=1, net%nin*net%nhid
                w1(i)=weights(j);
                j=j+1
	end do

!write(*,*) "tt_w1", w1
	do i=1, net%nout
                theta2(i)=weights(j);
                j=j+1
	end do
!write(*,*) "tt_theta2", theta2       
	do i=1, net%nhid*net%nout
                w2(i)=weights(j);
                j=j+1
        end do

!! apply feedforward calculation
!! hidden layer:
        do i=1, net%nhid
              hidden1(i)=theta1(i)
                do j=1, net%nin
                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + (i-1)*net%nin))
		end do
                hidden1(i)=tanh(hidden1(i))
        end do
!write(*,*) "tt_hidden1", hidden1
!! output layer
	do i=1, net%nout
                outputs(i)=theta2(i)
		do j=1, net%nhid
                        outputs(i) = outputs(i) + (hidden1(j)*w2(j+(i-1)*net%nhid))
		end do
	end do
!write(*,*) "tt_outputs", outputs
!! scale the results
	do i=1, net%nout
                MPK(i) = outputs(i)
	end do

!write(*,*) MPK

end subroutine 


subroutine regress_cambnet(P, weights, net, Cl, lmax, num_Cls, l)
use cmbtypes
integer i,j,  num_Cls, lmax
Type(NetStr) net
type(CAMBParams)  P
real Cl(lmax,1:num_Cls)
integer l(1:60)





real parameters(1:num_params_used)
real weights(1:net%nweights)
real scalein(1:net%nin)
real offin(1:net%nin)
real scaleout(1:net%nout)
real offout(1:net%nout)
real inputs(1:net%nin)
real hidden1(1:net%nhid)
real outputs(1:net%nout)
real theta1(1:net%nhid)
real theta2(1:net%nout)
real w1(1:net%nin*net%nhid)
real w2(1:net%nhid*net%nout)



do i=1, 10
parameters(i) = COSMONETP(i)
end do


      j=1
        do i=1, net%nhid
                theta1(i)=weights(j);
                j=j+1
	end do
!write(*,*) "tt_theta1", theta1

        do i=1, net%nin*net%nhid
                w1(i)=weights(j);
                j=j+1
	end do

!write(*,*) "tt_w1", w1
	do i=1, net%nout
                theta2(i)=weights(j);
                j=j+1
	end do
!write(*,*) "tt_theta2", theta2       
	do i=1, net%nhid*net%nout
                w2(i)=weights(j);
                j=j+1
        end do

!! apply feedforward calculation
!! hidden layer:
        do i=1, net%nhid
              hidden1(i)=theta1(i)
                do j=1, net%nin
                        hidden1(i) = hidden1(i) + (parameters(j)*w1(j + (i-1)*net%nin))
		end do
                hidden1(i)=tanh(hidden1(i))
        end do
!write(*,*) "tt_hidden1", hidden1
!! output layer
	do i=1, net%nout
                outputs(i)=theta2(i)
		do j=1, net%nhid
                        outputs(i) = outputs(i) + (hidden1(j)*w2(j+(i-1)*net%nhid))
		end do
	end do
!write(*,*) "tt_outputs", outputs
!! scale the results
	do i=1, net%nout
                Cl(i, num_Cls) = outputs(i)
		Cl(i,num_Cls) = 2*3.14159*Cl(i,num_Cls)/(l(i)*(l(i)+1))
		!Cl(i,num_Cls) = Cl(i,num_Cls)/((l(i)*(l(i)+1))*2*3.14159)
		!write(*,*) l(i), Cl(i,1), Cl(i,2), Cl(i,3)
	end do

!write(*,*) Cl

end subroutine 






end module cosmonet

